@@ -50,10 +50,12 @@
    ;; SLOT-VALUE and a LOOP.
    (ffi-name-transformer :initarg :ffi-name-transformer
                          :initform 'default-ffi-name-transformer)
+   (ffi-name-export-predicate :initarg :ffi-name-export-predicate
+                              :initform 'default-ffi-name-export-predicate)
    (ffi-type-transformer :initarg :ffi-type-transformer
                          :initform 'default-ffi-type-transformer)
-   (ffi-export-predicate :initarg :ffi-export-predicate
-                         :initform 'default-ffi-export-predicate)
+   (callback-factory :initarg :callback-factory
+                     :initform 'default-callback-factory)
    (foreign-library-name :initarg :foreign-library-name
                          :initform nil)
    (foreign-library-spec :initarg :foreign-library-spec
@@ -71,24 +73,36 @@
   (:default-initargs
    :type nil)
   (:documentation
-   "The input of this ASDF component is a C header file, plus configuration for the
-binding generation process. This header file will define the initial scope of
-the generation process, which can further be filtered by other configuration
+   "The input of this ASDF component is a C header file and the configuration for
+the binding generation process. This header file will define the initial scope of
+the generation process, which can be further filtered by other configuration
 parameters.
 
-An external program based on clang/llvm called 'c2ffi' is then invoked with
-this header file to generate a json file for various platforms. The generation
-of the underlying platform's json file must succeed to continue, but the
-generation for the other platforms is allowed to fail
-\(see ENSURE-SPEC-FILE-EXISTS).
+A clang/llvm based external program called 'c2ffi' is used to process this header
+file and generate a json spec file for each supported architecture triplet. Normally
+these .spec files are only (re)generated by the author of the lib and are checked into
+the corresponding source repository. It needs to be done manually by invoking the
+following command:
 
-It's advisable to deliver these json files with the project, so that users
-don't need to have c2ffi installed.
+(cffi/c2ffi:generate-spec :your-system)
 
-Once/if the json file is available for the underlying platform, then the json
-file is used to generate a lisp file with CFFI definitions (see
-PROCESS-C2FFI-SPEC-FILE). This file will then be compiled as any other lisp
-file, except that it's will be stored in the fasl cache."))
+which is a shorthand for:
+
+(asdf:operate 'cffi/c2ffi::generate-spec-op :your-system)
+
+The generation of the underlying platform's json file must succeed, but the
+generation for the other arch's is allowed to fail
+\(see ENSURE-SPEC-FILE-IS-UP-TO-DATE for details).
+
+During the normal build process the json file is used as the input to generate
+a lisp file containing the CFFI definitions (see PROCESS-C2FFI-SPEC-FILE).
+This file will be placed next to the .spec file, and will be compiled as any
+other lisp file. This process requires loading the ASDF system called
+\"cffi/c2ffi-generator\" that has more dependencies than CFFI itself. If you
+want to avoid those extra dependencies in your project, then you can check in
+these generated lisp files into your source repository, but keep in mind that
+you'll need to manually force their regeneration if CFFI/C2FFI itself gets
+updated (by e.g. deleting them from the filesystem) ."))
 
 (defun input-file (operation component)
   (let ((files (input-files operation component)))
@@ -98,6 +112,9 @@ file, except that it's will be stored in the fasl cache."))
 (defclass generate-spec-op (downward-operation)
   ())
 
+(defun generate-spec (system)
+  (asdf:operate 'generate-spec-op system))
+
 (defmethod input-files ((op generate-spec-op) (c c2ffi-file))
   (list (component-pathname c)))
 
@@ -112,6 +129,9 @@ file, except that it's will be stored in the fasl cache."))
      ;; Tell ASDF not to apply output translation.
      t)))
 
+(defmethod perform ((op generate-spec-op) (c asdf:component))
+  (values))
+
 (defmethod perform ((op generate-spec-op) (c c2ffi-file))
   (let ((input-file (input-file op c))
         (*c2ffi-executable* (if (slot-boundp c 'c2ffi-executable)
@@ -123,15 +143,17 @@ file, except that it's will be stored in the fasl cache."))
     ;; NOTE: we don't call OUTPUT-FILE here, which may be a violation
     ;; of the ASDF contract, that promises that OUTPUT-FILE can be
     ;; customized by users.
-    (ensure-spec-file-exists input-file
-                             :exclude-archs (c2ffi-file/exclude-archs c)
-                             :sys-include-paths (c2ffi-file/sys-include-paths c))))
+    (ensure-spec-file-is-up-to-date
+     input-file
+     :exclude-archs (c2ffi-file/exclude-archs c)
+     :sys-include-paths (c2ffi-file/sys-include-paths c))))
 
 (defclass generate-lisp-op (downward-operation)
   ())
 
 (defmethod component-depends-on ((op generate-lisp-op) (c c2ffi-file))
-  `((generate-spec-op ,c) ,@(call-next-method)))
+  `((load-op ,(find-system "cffi/c2ffi-generator"))
+    ,@(call-next-method)))
 
 (defmethod component-depends-on ((op compile-op) (c c2ffi-file))
   `((generate-lisp-op ,c) ,@(call-next-method)))
@@ -159,8 +181,6 @@ file, except that it's will be stored in the fasl cache."))
         (generated-lisp-file (output-file op c)))
     (with-staging-pathname (tmp-output generated-lisp-file)
       (format *debug-io* "~&; CFFI/C2FFI is generating the file ~S~%" generated-lisp-file)
-      (unless (component-loaded-p :cffi/c2ffi-generator)
-        (load-system :cffi/c2ffi-generator))
       (apply 'process-c2ffi-spec-file
              spec-file (c2ffi-file/package c)
              :output tmp-output
@@ -174,8 +194,9 @@ file, except that it's will be stored in the fasl cache."))
              ;; component and in PROCESS-C2FFI-SPEC-FILE, and this loop copies them.
              (loop
                :for arg :in '(ffi-name-transformer
+                              ffi-name-export-predicate
                               ffi-type-transformer
-                              ffi-export-predicate
+                              callback-factory
                               foreign-library-name
                               foreign-library-spec
                               emit-generated-name-mappings
