@@ -208,8 +208,9 @@ to open-code (SETF MEM-REF) forms."
         #+cffi-sys::no-long-long
         (when (member ctype '(:long-long :unsigned-long-long))
           (return-from mem-set form))
-        (if (aggregatep parsed-type)    ; XXX: skip for now.
-            form      ; use expand-into-foreign-memory when available.
+        (if (aggregatep parsed-type)
+            (expand-into-foreign-memory
+             value parsed-type `(inc-pointer ,ptr ,offset))
             `(%mem-set ,(expand-to-foreign value parsed-type)
                        ,ptr ,ctype ,offset)))
       form))
@@ -387,7 +388,8 @@ newly allocated memory."
       form))
 
 (defun lisp-array-to-foreign (array pointer array-type)
-  "Copy elements from a Lisp array to POINTER."
+  "Copy elements from a Lisp array to POINTER. ARRAY-TYPE must be a CFFI array
+type."
   (let* ((type (ensure-parsed-base-type array-type))
          (el-type (element-type type))
          (dimensions (dimensions type)))
@@ -399,14 +401,15 @@ newly allocated memory."
                                (row-major-index-to-indexes i dimensions))
           do (setf (mem-ref pointer el-type offset) element))))
 
-(defun foreign-array-to-lisp (pointer array-type)
-  "Copy elements from ptr into a Lisp array. If POINTER is a null
-pointer, returns NIL."
+(defun foreign-array-to-lisp (pointer array-type &rest make-array-args)
+  "Copy elements from pointer into a Lisp array. ARRAY-TYPE must be a CFFI array
+type; the type of the resulting Lisp array can be defined in MAKE-ARRAY-ARGS
+that are then passed to MAKE-ARRAY. If POINTER is a null pointer, returns NIL."
   (unless (null-pointer-p pointer)
     (let* ((type (ensure-parsed-base-type array-type))
            (el-type (element-type type))
            (dimensions (dimensions type))
-           (array (make-array dimensions)))
+           (array (apply #'make-array dimensions make-array-args)))
       (loop with foreign-type-size = (array-element-size type)
             with size = (reduce #'* dimensions)
             for i from 0 below size
@@ -764,6 +767,12 @@ The foreign array must be freed with foreign-array-free."
   "Return the address of SLOT-NAME in the structure at PTR."
   (foreign-struct-slot-pointer ptr (get-slot-info type slot-name)))
 
+(define-compiler-macro foreign-slot-pointer (&whole whole ptr type slot-name)
+  (if (and (constantp type) (constantp slot-name))
+      (foreign-struct-slot-pointer-form
+       ptr (get-slot-info (eval type) (eval slot-name)))
+      whole))
+
 (defun foreign-slot-type (type slot-name)
   "Return the type of SLOT in a struct TYPE."
   (slot-type (get-slot-info type slot-name)))
